# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class ProcessorResponseCode(object):

    """Implementation of the 'Processor Response Code' enum.

    Processor response code for the non-PayPal payment processor errors.

    Attributes:
        RESPONSE_0000: APPROVED.
        RESPONSE_00N7: CVV2_FAILURE_POSSIBLE_RETRY_WITH_CVV.
        RESPONSE_0100: REFERRAL.
        RESPONSE_0390: ACCOUNT_NOT_FOUND.
        RESPONSE_0500: DO_NOT_HONOR.
        RESPONSE_0580: UNAUTHORIZED_TRANSACTION.
        RESPONSE_0800: BAD_RESPONSE_REVERSAL_REQUIRED.
        RESPONSE_0880: CRYPTOGRAPHIC_FAILURE.
        RESPONSE_0890: UNACCEPTABLE_PIN.
        RESPONSE_0960: SYSTEM_MALFUNCTION.
        RESPONSE_0R00: CANCELLED_PAYMENT.
        RESPONSE_1000: PARTIAL_AUTHORIZATION.
        RESPONSE_10BR: ISSUER_REJECTED.
        RESPONSE_1300: INVALID_DATA_FORMAT.
        RESPONSE_1310: INVALID_AMOUNT.
        RESPONSE_1312: INVALID_TRANSACTION_CARD_ISSUER_ACQUIRER.
        RESPONSE_1317: INVALID_CAPTURE_DATE.
        RESPONSE_1320: INVALID_CURRENCY_CODE.
        RESPONSE_1330: INVALID_ACCOUNT.
        RESPONSE_1335: INVALID_ACCOUNT_RECURRING.
        RESPONSE_1340: INVALID_TERMINAL.
        RESPONSE_1350: INVALID_MERCHANT.
        RESPONSE_1352: RESTRICTED_OR_INACTIVE_ACCOUNT.
        RESPONSE_1360: BAD_PROCESSING_CODE.
        RESPONSE_1370: INVALID_MCC.
        RESPONSE_1380: INVALID_EXPIRATION.
        RESPONSE_1382: INVALID_CARD_VERIFICATION_VALUE.
        RESPONSE_1384: INVALID_LIFE_CYCLE_OF_TRANSACTION.
        RESPONSE_1390: INVALID_ORDER.
        RESPONSE_1393: TRANSACTION_CANNOT_BE_COMPLETED.
        RESPONSE_5100: GENERIC_DECLINE.
        RESPONSE_5110: CVV2_FAILURE.
        RESPONSE_5120: INSUFFICIENT_FUNDS.
        RESPONSE_5130: INVALID_PIN.
        RESPONSE_5135: DECLINED_PIN_TRY_EXCEEDED.
        RESPONSE_5140: CARD_CLOSED.
        RESPONSE_5150: PICKUP_CARD_SPECIAL_CONDITIONS. Try using another card.
            Do not retry the same card.
        RESPONSE_5160: UNAUTHORIZED_USER.
        RESPONSE_5170: AVS_FAILURE.
        RESPONSE_5180: INVALID_OR_RESTRICTED_CARD. Try using another card. Do
            not retry the same card.
        RESPONSE_5190: SOFT_AVS.
        RESPONSE_5200: DUPLICATE_TRANSACTION.
        RESPONSE_5210: INVALID_TRANSACTION.
        RESPONSE_5400: EXPIRED_CARD.
        RESPONSE_5500: INCORRECT_PIN_REENTER.
        RESPONSE_5650: DECLINED_SCA_REQUIRED.
        RESPONSE_5700: TRANSACTION_NOT_PERMITTED. Outside of scope of accepted
            business.
        RESPONSE_5710: TX_ATTEMPTS_EXCEED_LIMIT.
        RESPONSE_5800: REVERSAL_REJECTED.
        RESPONSE_5900: INVALID_ISSUE.
        RESPONSE_5910: ISSUER_NOT_AVAILABLE_NOT_RETRIABLE.
        RESPONSE_5920: ISSUER_NOT_AVAILABLE_RETRIABLE.
        RESPONSE_5930: CARD_NOT_ACTIVATED.
        RESPONSE_5950: DECLINED_DUE_TO_UPDATED_ACCOUNT. External decline as an
            updated card has been issued.
        RESPONSE_6300: ACCOUNT_NOT_ON_FILE.
        RESPONSE_7600: APPROVED_NON_CAPTURE.
        RESPONSE_7700: ERROR_3DS.
        RESPONSE_7710: AUTHENTICATION_FAILED.
        RESPONSE_7800: BIN_ERROR.
        RESPONSE_7900: PIN_ERROR.
        RESPONSE_8000: PROCESSOR_SYSTEM_ERROR.
        RESPONSE_8010: HOST_KEY_ERROR.
        RESPONSE_8020: CONFIGURATION_ERROR.
        RESPONSE_8030: UNSUPPORTED_OPERATION.
        RESPONSE_8100: FATAL_COMMUNICATION_ERROR.
        RESPONSE_8110: RETRIABLE_COMMUNICATION_ERROR.
        RESPONSE_8220: SYSTEM_UNAVAILABLE.
        RESPONSE_9100: DECLINED_PLEASE_RETRY. Retry.
        RESPONSE_9500: SUSPECTED_FRAUD. Try using another card. Do not retry
            the same card.
        RESPONSE_9510: SECURITY_VIOLATION.
        RESPONSE_9520: LOST_OR_STOLEN. Try using another card. Do not retry
            the same card.
        RESPONSE_9530: HOLD_CALL_CENTER. The merchant must call the number on
            the back of the card. POS scenario.
        RESPONSE_9540: REFUSED_CARD.
        RESPONSE_9600: UNRECOGNIZED_RESPONSE_CODE.
        RESPONSE_PCNR: CONTINGENCIES_NOT_RESOLVED.
        RESPONSE_PCVV: CVV_FAILURE.
        RESPONSE_PP06: ACCOUNT_CLOSED. A previously open account is now closed
        RESPONSE_PPRN: REATTEMPT_NOT_PERMITTED.
        RESPONSE_PPAD: BILLING_ADDRESS.
        RESPONSE_PPAB: ACCOUNT_BLOCKED_BY_ISSUER.
        RESPONSE_PPAE: AMEX_DISABLED.
        RESPONSE_PPAG: ADULT_GAMING_UNSUPPORTED.
        RESPONSE_PPAI: AMOUNT_INCOMPATIBLE.
        RESPONSE_PPAR: AUTH_RESULT.
        RESPONSE_PPAU: MCC_CODE.
        RESPONSE_PPAV: ARC_AVS.
        RESPONSE_PPAX: AMOUNT_EXCEEDED.
        RESPONSE_PPBG: BAD_GAMING.
        RESPONSE_PPC2: ARC_CVV.
        RESPONSE_PPCE: CE_REGISTRATION_INCOMPLETE.
        RESPONSE_PPCO: COUNTRY.
        RESPONSE_PPCR: CREDIT_ERROR.
        RESPONSE_PPCT: CARD_TYPE_UNSUPPORTED.
        RESPONSE_PPCU: CURRENCY_USED_INVALID.
        RESPONSE_PPD3: SECURE_ERROR_3DS.
        RESPONSE_PPDC: DCC_UNSUPPORTED.
        RESPONSE_PPDI: DINERS_REJECT.
        RESPONSE_PPDV: AUTH_MESSAGE.
        RESPONSE_PPDT: DECLINE_THRESHOLD_BREACH.
        RESPONSE_PPEF: EXPIRED_FUNDING_INSTRUMENT.
        RESPONSE_PPEL: EXCEEDS_FREQUENCY_LIMIT.
        RESPONSE_PPER: INTERNAL_SYSTEM_ERROR.
        RESPONSE_PPEX: EXPIRY_DATE.
        RESPONSE_PPFE: FUNDING_SOURCE_ALREADY_EXISTS.
        RESPONSE_PPFI: INVALID_FUNDING_INSTRUMENT.
        RESPONSE_PPFR: RESTRICTED_FUNDING_INSTRUMENT.
        RESPONSE_PPFV: FIELD_VALIDATION_FAILED.
        RESPONSE_PPGR: GAMING_REFUND_ERROR.
        RESPONSE_PPH1: H1_ERROR.
        RESPONSE_PPIF: IDEMPOTENCY_FAILURE.
        RESPONSE_PPII: INVALID_INPUT_FAILURE.
        RESPONSE_PPIM: ID_MISMATCH.
        RESPONSE_PPIT: INVALID_TRACE_ID.
        RESPONSE_PPLR: LATE_REVERSAL.
        RESPONSE_PPLS: LARGE_STATUS_CODE.
        RESPONSE_PPMB: MISSING_BUSINESS_RULE_OR_DATA.
        RESPONSE_PPMC: BLOCKED_Mastercard.
        RESPONSE_PPMD: DEPRECATED The PPMD value has been deprecated.
        RESPONSE_PPNC: NOT_SUPPORTED_NRC.
        RESPONSE_PPNL: EXCEEDS_NETWORK_FREQUENCY_LIMIT.
        RESPONSE_PPNM: NO_MID_FOUND.
        RESPONSE_PPNT: NETWORK_ERROR.
        RESPONSE_PPPH: NO_PHONE_FOR_DCC_TRANSACTION.
        RESPONSE_PPPI: INVALID_PRODUCT.
        RESPONSE_PPPM: INVALID_PAYMENT_METHOD.
        RESPONSE_PPQC: QUASI_CASH_UNSUPPORTED.
        RESPONSE_PPRE: UNSUPPORT_REFUND_ON_PENDING_BC.
        RESPONSE_PPRF: INVALID_PARENT_TRANSACTION_STATUS.
        RESPONSE_PPRR: MERCHANT_NOT_REGISTERED.
        RESPONSE_PPS0: BANKAUTH_ROW_MISMATCH.
        RESPONSE_PPS1: BANKAUTH_ROW_SETTLED.
        RESPONSE_PPS2: BANKAUTH_ROW_VOIDED.
        RESPONSE_PPS3: BANKAUTH_EXPIRED.
        RESPONSE_PPS4: CURRENCY_MISMATCH.
        RESPONSE_PPS5: CREDITCARD_MISMATCH.
        RESPONSE_PPS6: AMOUNT_MISMATCH.
        RESPONSE_PPSC: ARC_SCORE.
        RESPONSE_PPSD: STATUS_DESCRIPTION.
        RESPONSE_PPSE: AMEX_DENIED.
        RESPONSE_PPTE: VERIFICATION_TOKEN_EXPIRED.
        RESPONSE_PPTF: INVALID_TRACE_REFERENCE.
        RESPONSE_PPTI: INVALID_TRANSACTION_ID.
        RESPONSE_PPTR: VERIFICATION_TOKEN_REVOKED.
        RESPONSE_PPTT: TRANSACTION_TYPE_UNSUPPORTED.
        RESPONSE_PPTV: INVALID_VERIFICATION_TOKEN.
        RESPONSE_PPUA: USER_NOT_AUTHORIZED.
        RESPONSE_PPUC: CURRENCY_CODE_UNSUPPORTED.
        RESPONSE_PPUE: UNSUPPORT_ENTITY.
        RESPONSE_PPUI: UNSUPPORT_INSTALLMENT.
        RESPONSE_PPUP: UNSUPPORT_POS_FLAG.
        RESPONSE_PPUR: UNSUPPORTED_REVERSAL.
        RESPONSE_PPVC: VALIDATE_CURRENCY.
        RESPONSE_PPVE: VALIDATION_ERROR.
        RESPONSE_PPVT: VIRTUAL_TERMINAL_UNSUPPORTED.

    """
    RESPONSE_0000 = '0000'

    RESPONSE_00N7 = '00N7'

    RESPONSE_0100 = '0100'

    RESPONSE_0390 = '0390'

    RESPONSE_0500 = '0500'

    RESPONSE_0580 = '0580'

    RESPONSE_0800 = '0800'

    RESPONSE_0880 = '0880'

    RESPONSE_0890 = '0890'

    RESPONSE_0960 = '0960'

    RESPONSE_0R00 = '0R00'

    RESPONSE_1000 = '1000'

    RESPONSE_10BR = '10BR'

    RESPONSE_1300 = '1300'

    RESPONSE_1310 = '1310'

    RESPONSE_1312 = '1312'

    RESPONSE_1317 = '1317'

    RESPONSE_1320 = '1320'

    RESPONSE_1330 = '1330'

    RESPONSE_1335 = '1335'

    RESPONSE_1340 = '1340'

    RESPONSE_1350 = '1350'

    RESPONSE_1352 = '1352'

    RESPONSE_1360 = '1360'

    RESPONSE_1370 = '1370'

    RESPONSE_1380 = '1380'

    RESPONSE_1382 = '1382'

    RESPONSE_1384 = '1384'

    RESPONSE_1390 = '1390'

    RESPONSE_1393 = '1393'

    RESPONSE_5100 = '5100'

    RESPONSE_5110 = '5110'

    RESPONSE_5120 = '5120'

    RESPONSE_5130 = '5130'

    RESPONSE_5135 = '5135'

    RESPONSE_5140 = '5140'

    RESPONSE_5150 = '5150'

    RESPONSE_5160 = '5160'

    RESPONSE_5170 = '5170'

    RESPONSE_5180 = '5180'

    RESPONSE_5190 = '5190'

    RESPONSE_5200 = '5200'

    RESPONSE_5210 = '5210'

    RESPONSE_5400 = '5400'

    RESPONSE_5500 = '5500'

    RESPONSE_5650 = '5650'

    RESPONSE_5700 = '5700'

    RESPONSE_5710 = '5710'

    RESPONSE_5800 = '5800'

    RESPONSE_5900 = '5900'

    RESPONSE_5910 = '5910'

    RESPONSE_5920 = '5920'

    RESPONSE_5930 = '5930'

    RESPONSE_5950 = '5950'

    RESPONSE_6300 = '6300'

    RESPONSE_7600 = '7600'

    RESPONSE_7700 = '7700'

    RESPONSE_7710 = '7710'

    RESPONSE_7800 = '7800'

    RESPONSE_7900 = '7900'

    RESPONSE_8000 = '8000'

    RESPONSE_8010 = '8010'

    RESPONSE_8020 = '8020'

    RESPONSE_8030 = '8030'

    RESPONSE_8100 = '8100'

    RESPONSE_8110 = '8110'

    RESPONSE_8220 = '8220'

    RESPONSE_9100 = '9100'

    RESPONSE_9500 = '9500'

    RESPONSE_9510 = '9510'

    RESPONSE_9520 = '9520'

    RESPONSE_9530 = '9530'

    RESPONSE_9540 = '9540'

    RESPONSE_9600 = '9600'

    RESPONSE_PCNR = 'PCNR'

    RESPONSE_PCVV = 'PCVV'

    RESPONSE_PP06 = 'PP06'

    RESPONSE_PPRN = 'PPRN'

    RESPONSE_PPAD = 'PPAD'

    RESPONSE_PPAB = 'PPAB'

    RESPONSE_PPAE = 'PPAE'

    RESPONSE_PPAG = 'PPAG'

    RESPONSE_PPAI = 'PPAI'

    RESPONSE_PPAR = 'PPAR'

    RESPONSE_PPAU = 'PPAU'

    RESPONSE_PPAV = 'PPAV'

    RESPONSE_PPAX = 'PPAX'

    RESPONSE_PPBG = 'PPBG'

    RESPONSE_PPC2 = 'PPC2'

    RESPONSE_PPCE = 'PPCE'

    RESPONSE_PPCO = 'PPCO'

    RESPONSE_PPCR = 'PPCR'

    RESPONSE_PPCT = 'PPCT'

    RESPONSE_PPCU = 'PPCU'

    RESPONSE_PPD3 = 'PPD3'

    RESPONSE_PPDC = 'PPDC'

    RESPONSE_PPDI = 'PPDI'

    RESPONSE_PPDV = 'PPDV'

    RESPONSE_PPDT = 'PPDT'

    RESPONSE_PPEF = 'PPEF'

    RESPONSE_PPEL = 'PPEL'

    RESPONSE_PPER = 'PPER'

    RESPONSE_PPEX = 'PPEX'

    RESPONSE_PPFE = 'PPFE'

    RESPONSE_PPFI = 'PPFI'

    RESPONSE_PPFR = 'PPFR'

    RESPONSE_PPFV = 'PPFV'

    RESPONSE_PPGR = 'PPGR'

    RESPONSE_PPH1 = 'PPH1'

    RESPONSE_PPIF = 'PPIF'

    RESPONSE_PPII = 'PPII'

    RESPONSE_PPIM = 'PPIM'

    RESPONSE_PPIT = 'PPIT'

    RESPONSE_PPLR = 'PPLR'

    RESPONSE_PPLS = 'PPLS'

    RESPONSE_PPMB = 'PPMB'

    RESPONSE_PPMC = 'PPMC'

    RESPONSE_PPMD = 'PPMD'

    RESPONSE_PPNC = 'PPNC'

    RESPONSE_PPNL = 'PPNL'

    RESPONSE_PPNM = 'PPNM'

    RESPONSE_PPNT = 'PPNT'

    RESPONSE_PPPH = 'PPPH'

    RESPONSE_PPPI = 'PPPI'

    RESPONSE_PPPM = 'PPPM'

    RESPONSE_PPQC = 'PPQC'

    RESPONSE_PPRE = 'PPRE'

    RESPONSE_PPRF = 'PPRF'

    RESPONSE_PPRR = 'PPRR'

    RESPONSE_PPS0 = 'PPS0'

    RESPONSE_PPS1 = 'PPS1'

    RESPONSE_PPS2 = 'PPS2'

    RESPONSE_PPS3 = 'PPS3'

    RESPONSE_PPS4 = 'PPS4'

    RESPONSE_PPS5 = 'PPS5'

    RESPONSE_PPS6 = 'PPS6'

    RESPONSE_PPSC = 'PPSC'

    RESPONSE_PPSD = 'PPSD'

    RESPONSE_PPSE = 'PPSE'

    RESPONSE_PPTE = 'PPTE'

    RESPONSE_PPTF = 'PPTF'

    RESPONSE_PPTI = 'PPTI'

    RESPONSE_PPTR = 'PPTR'

    RESPONSE_PPTT = 'PPTT'

    RESPONSE_PPTV = 'PPTV'

    RESPONSE_PPUA = 'PPUA'

    RESPONSE_PPUC = 'PPUC'

    RESPONSE_PPUE = 'PPUE'

    RESPONSE_PPUI = 'PPUI'

    RESPONSE_PPUP = 'PPUP'

    RESPONSE_PPUR = 'PPUR'

    RESPONSE_PPVC = 'PPVC'

    RESPONSE_PPVE = 'PPVE'

    RESPONSE_PPVT = 'PPVT'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class StoredPaymentSourcePaymentType(object):

    """Implementation of the 'Stored Payment Source Payment Type' enum.

    Indicates the type of the stored payment_source payment.

    Attributes:
        ONE_TIME: One Time payment such as online purchase or donation. (e.g.
            Checkout with one-click).
        RECURRING: Payment which is part of a series of payments with fixed or
            variable amounts, following a fixed time interval. (e.g.
            Subscription payments).
        UNSCHEDULED: Payment which is part of a series of payments that occur
            on a non-fixed schedule and/or have variable amounts. (e.g.
            Account Topup payments).

    """
    ONE_TIME = 'ONE_TIME'

    RECURRING = 'RECURRING'

    UNSCHEDULED = 'UNSCHEDULED'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class PayeePaymentMethodPreference(object):

    """Implementation of the 'Payee Payment Method Preference' enum.

    The merchant-preferred payment methods.

    Attributes:
        UNRESTRICTED: Accepts any type of payment from the customer.
        IMMEDIATE_PAYMENT_REQUIRED: Accepts only immediate payment from the
            customer. For example, credit card, PayPal balance, or instant
            ACH. Ensures that at the time of capture, the payment does not
            have the `pending` status.

    """
    UNRESTRICTED = 'UNRESTRICTED'

    IMMEDIATE_PAYMENT_REQUIRED = 'IMMEDIATE_PAYMENT_REQUIRED'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper


class NativeAppContext(object):

    """Implementation of the 'Native App Context' model.

    Merchant provided, buyer's native app preferences to app switch to the
    PayPal consumer app.

    Attributes:
        os_type (OsType): Operating System type of the device that the buyer
            is using.
        os_version (str): Operating System version of the device that the
            buyer is using.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "os_type": 'os_type',
        "os_version": 'os_version'
    }

    _optionals = [
        'os_type',
        'os_version',
    ]

    def __init__(self,
                 os_type=APIHelper.SKIP,
                 os_version=APIHelper.SKIP):
        """Constructor for the NativeAppContext class"""

        # Initialize members of the class
        if os_type is not APIHelper.SKIP:
            self.os_type = os_type 
        if os_version is not APIHelper.SKIP:
            self.os_version = os_version 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        os_type = dictionary.get("os_type") if dictionary.get("os_type") else APIHelper.SKIP
        os_version = dictionary.get("os_version") if dictionary.get("os_version") else APIHelper.SKIP
        # Return an object of this model
        return cls(os_type,
                   os_version)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'os_type={(self.os_type if hasattr(self, "os_type") else None)!r}, '
                f'os_version={(self.os_version if hasattr(self, "os_version") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'os_type={(self.os_type if hasattr(self, "os_type") else None)!s}, '
                f'os_version={(self.os_version if hasattr(self, "os_version") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class ExperienceStatus(object):

    """Implementation of the 'Experience Status' enum.

    This field indicates the status of PayPal's Checkout experience throughout
    the order lifecycle. The values reflect the current stage of the checkout
    process.

    Attributes:
        NOT_STARTED: PayPal checkout process has not yet begun.
        IN_PROGRESS: PayPal checkout initiated. User is on the checkout page
            for order review before approval.
        CANCELED: PayPal checkout is canceled (by closing the checkout window
            or clicking cancel) before the order approval.
        APPROVED: Order is approved. User has completed the checkout process.

    """
    NOT_STARTED = 'NOT_STARTED'

    IN_PROGRESS = 'IN_PROGRESS'

    CANCELED = 'CANCELED'

    APPROVED = 'APPROVED'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.incentive_details import IncentiveDetails


class IncentiveInformation(object):

    """Implementation of the 'Incentive Information' model.

    The incentive details.

    Attributes:
        incentive_details (List[IncentiveDetails]): An array of incentive
            details.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "incentive_details": 'incentive_details'
    }

    _optionals = [
        'incentive_details',
    ]

    def __init__(self,
                 incentive_details=APIHelper.SKIP):
        """Constructor for the IncentiveInformation class"""

        # Initialize members of the class
        if incentive_details is not APIHelper.SKIP:
            self.incentive_details = incentive_details 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        incentive_details = None
        if dictionary.get('incentive_details') is not None:
            incentive_details = [IncentiveDetails.from_dictionary(x) for x in dictionary.get('incentive_details')]
        else:
            incentive_details = APIHelper.SKIP
        # Return an object of this model
        return cls(incentive_details)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'incentive_details={(self.incentive_details if hasattr(self, "incentive_details") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'incentive_details={(self.incentive_details if hasattr(self, "incentive_details") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.money import Money


class TaxAmount(object):

    """Implementation of the 'Tax Amount' model.

    The tax levied by a government on the purchase of goods or services.

    Attributes:
        tax_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "tax_amount": 'tax_amount'
    }

    _optionals = [
        'tax_amount',
    ]

    def __init__(self,
                 tax_amount=APIHelper.SKIP):
        """Constructor for the TaxAmount class"""

        # Initialize members of the class
        if tax_amount is not APIHelper.SKIP:
            self.tax_amount = tax_amount 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        tax_amount = Money.from_dictionary(dictionary.get('tax_amount')) if 'tax_amount' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(tax_amount)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'tax_amount={(self.tax_amount if hasattr(self, "tax_amount") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'tax_amount={(self.tax_amount if hasattr(self, "tax_amount") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.money import Money


class PricingTier(object):

    """Implementation of the 'Pricing Tier' model.

    The pricing tier details.

    Attributes:
        starting_quantity (str): The starting quantity for the tier.
        ending_quantity (str): The ending quantity for the tier. Optional for
            the last tier.
        amount (Money): The currency and amount for a financial transaction,
            such as a balance or payment due.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "starting_quantity": 'starting_quantity',
        "amount": 'amount',
        "ending_quantity": 'ending_quantity'
    }

    _optionals = [
        'ending_quantity',
    ]

    def __init__(self,
                 starting_quantity=None,
                 amount=None,
                 ending_quantity=APIHelper.SKIP):
        """Constructor for the PricingTier class"""

        # Initialize members of the class
        self.starting_quantity = starting_quantity 
        if ending_quantity is not APIHelper.SKIP:
            self.ending_quantity = ending_quantity 
        self.amount = amount 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        starting_quantity = dictionary.get("starting_quantity") if dictionary.get("starting_quantity") else None
        amount = Money.from_dictionary(dictionary.get('amount')) if dictionary.get('amount') else None
        ending_quantity = dictionary.get("ending_quantity") if dictionary.get("ending_quantity") else APIHelper.SKIP
        # Return an object of this model
        return cls(starting_quantity,
                   amount,
                   ending_quantity)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'starting_quantity={self.starting_quantity!r}, '
                f'ending_quantity={(self.ending_quantity if hasattr(self, "ending_quantity") else None)!r}, '
                f'amount={self.amount!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'starting_quantity={self.starting_quantity!s}, '
                f'ending_quantity={(self.ending_quantity if hasattr(self, "ending_quantity") else None)!s}, '
                f'amount={self.amount!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.mobile_web_context import MobileWebContext
from paypalserversdk.models.native_app_context import NativeAppContext


class AppSwitchContext(object):

    """Implementation of the 'App Switch Context' model.

    Merchant provided details of the native app or mobile web browser to
    facilitate buyer's app switch to the PayPal consumer app.

    Attributes:
        native_app (NativeAppContext): Merchant provided, buyer's native app
            preferences to app switch to the PayPal consumer app.
        mobile_web (MobileWebContext): Buyer's mobile web browser context to
            app switch to the PayPal consumer app.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "native_app": 'native_app',
        "mobile_web": 'mobile_web'
    }

    _optionals = [
        'native_app',
        'mobile_web',
    ]

    def __init__(self,
                 native_app=APIHelper.SKIP,
                 mobile_web=APIHelper.SKIP):
        """Constructor for the AppSwitchContext class"""

        # Initialize members of the class
        if native_app is not APIHelper.SKIP:
            self.native_app = native_app 
        if mobile_web is not APIHelper.SKIP:
            self.mobile_web = mobile_web 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        native_app = NativeAppContext.from_dictionary(dictionary.get('native_app')) if 'native_app' in dictionary.keys() else APIHelper.SKIP
        mobile_web = MobileWebContext.from_dictionary(dictionary.get('mobile_web')) if 'mobile_web' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(native_app,
                   mobile_web)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'native_app={(self.native_app if hasattr(self, "native_app") else None)!r}, '
                f'mobile_web={(self.mobile_web if hasattr(self, "mobile_web") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'native_app={(self.native_app if hasattr(self, "native_app") else None)!s}, '
                f'mobile_web={(self.mobile_web if hasattr(self, "mobile_web") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.merchant_preferences import MerchantPreferences
from paypalserversdk.models.payment_preferences import PaymentPreferences
from paypalserversdk.models.subscription_billing_cycle import SubscriptionBillingCycle
from paypalserversdk.models.taxes import Taxes


class PlanRequest(object):

    """Implementation of the 'Plan Request' model.

    The create plan request details.

    Attributes:
        product_id (str): The ID of the product created through Catalog
            Products API.
        name (str): The plan name.
        status (PlanRequestStatus): The initial state of the plan. Allowed
            input values are CREATED and ACTIVE.
        description (str): The detailed description of the plan.
        billing_cycles (List[SubscriptionBillingCycle]): An array of billing
            cycles for trial billing and regular billing. A plan can have at
            most two trial cycles and only one regular cycle.
        payment_preferences (PaymentPreferences): The payment preferences for
            a subscription.
        merchant_preferences (MerchantPreferences): The merchant preferences
            for a subscription.
        taxes (Taxes): The tax details.
        quantity_supported (bool): Indicates whether you can subscribe to this
            plan by providing a quantity for the goods or service.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "product_id": 'product_id',
        "name": 'name',
        "billing_cycles": 'billing_cycles',
        "payment_preferences": 'payment_preferences',
        "status": 'status',
        "description": 'description',
        "merchant_preferences": 'merchant_preferences',
        "taxes": 'taxes',
        "quantity_supported": 'quantity_supported'
    }

    _optionals = [
        'status',
        'description',
        'merchant_preferences',
        'taxes',
        'quantity_supported',
    ]

    def __init__(self,
                 product_id=None,
                 name=None,
                 billing_cycles=None,
                 payment_preferences=None,
                 status='ACTIVE',
                 description=APIHelper.SKIP,
                 merchant_preferences=APIHelper.SKIP,
                 taxes=APIHelper.SKIP,
                 quantity_supported=False):
        """Constructor for the PlanRequest class"""

        # Initialize members of the class
        self.product_id = product_id 
        self.name = name 
        self.status = status 
        if description is not APIHelper.SKIP:
            self.description = description 
        self.billing_cycles = billing_cycles 
        self.payment_preferences = payment_preferences 
        if merchant_preferences is not APIHelper.SKIP:
            self.merchant_preferences = merchant_preferences 
        if taxes is not APIHelper.SKIP:
            self.taxes = taxes 
        self.quantity_supported = quantity_supported 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        product_id = dictionary.get("product_id") if dictionary.get("product_id") else None
        name = dictionary.get("name") if dictionary.get("name") else None
        billing_cycles = None
        if dictionary.get('billing_cycles') is not None:
            billing_cycles = [SubscriptionBillingCycle.from_dictionary(x) for x in dictionary.get('billing_cycles')]
        payment_preferences = PaymentPreferences.from_dictionary(dictionary.get('payment_preferences')) if dictionary.get('payment_preferences') else None
        status = dictionary.get("status") if dictionary.get("status") else 'ACTIVE'
        description = dictionary.get("description") if dictionary.get("description") else APIHelper.SKIP
        merchant_preferences = MerchantPreferences.from_dictionary(dictionary.get('merchant_preferences')) if 'merchant_preferences' in dictionary.keys() else APIHelper.SKIP
        taxes = Taxes.from_dictionary(dictionary.get('taxes')) if 'taxes' in dictionary.keys() else APIHelper.SKIP
        quantity_supported = dictionary.get("quantity_supported") if dictionary.get("quantity_supported") else False
        # Return an object of this model
        return cls(product_id,
                   name,
                   billing_cycles,
                   payment_preferences,
                   status,
                   description,
                   merchant_preferences,
                   taxes,
                   quantity_supported)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'product_id={self.product_id!r}, '
                f'name={self.name!r}, '
                f'status={(self.status if hasattr(self, "status") else None)!r}, '
                f'description={(self.description if hasattr(self, "description") else None)!r}, '
                f'billing_cycles={self.billing_cycles!r}, '
                f'payment_preferences={self.payment_preferences!r}, '
                f'merchant_preferences={(self.merchant_preferences if hasattr(self, "merchant_preferences") else None)!r}, '
                f'taxes={(self.taxes if hasattr(self, "taxes") else None)!r}, '
                f'quantity_supported={(self.quantity_supported if hasattr(self, "quantity_supported") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'product_id={self.product_id!s}, '
                f'name={self.name!s}, '
                f'status={(self.status if hasattr(self, "status") else None)!s}, '
                f'description={(self.description if hasattr(self, "description") else None)!s}, '
                f'billing_cycles={self.billing_cycles!s}, '
                f'payment_preferences={self.payment_preferences!s}, '
                f'merchant_preferences={(self.merchant_preferences if hasattr(self, "merchant_preferences") else None)!s}, '
                f'taxes={(self.taxes if hasattr(self, "taxes") else None)!s}, '
                f'quantity_supported={(self.quantity_supported if hasattr(self, "quantity_supported") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.money import Money


class TransactionInformation(object):

    """Implementation of the 'Transaction Information' model.

    The transaction information.

    Attributes:
        paypal_account_id (str): The ID of the PayPal account of the
            counterparty.
        transaction_id (str): The PayPal-generated transaction ID.
        paypal_reference_id (str): The PayPal-generated base ID. PayPal
            exclusive. Cannot be altered. Defined as a related, pre-existing
            transaction or event.
        paypal_reference_id_type (PaypalReferenceIdType): The PayPal reference
            ID type.
        transaction_event_code (str): A five-digit transaction event code that
            classifies the transaction type based on money movement and debit
            or credit. For example, T0001. See [Transaction event
            codes](/docs/integration/direct/transaction-search/transaction-even
            t-codes/).
        transaction_initiation_date (str): The date and time, in [Internet
            date and time
            format](https://tools.ietf.org/html/rfc3339#section-5.6). Seconds
            are required while fractional seconds are optional. Note: The
            regular expression provides guidance but does not reject all
            invalid dates.
        transaction_updated_date (str): The date and time, in [Internet date
            and time format](https://tools.ietf.org/html/rfc3339#section-5.6).
            Seconds are required while fractional seconds are optional. Note:
            The regular expression provides guidance but does not reject all
            invalid dates.
        transaction_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        fee_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        discount_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        insurance_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        sales_tax_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        shipping_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        shipping_discount_amount (Money): The currency and amount for a
            financial transaction, such as a balance or payment due.
        shipping_tax_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        other_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        tip_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        transaction_status (str): A code that indicates the transaction
            status. Value is: Status code Description D PayPal or merchant
            rules denied the transaction. P The transaction is pending. The
            transaction was created but waits for another payment process to
            complete, such as an ACH transaction, before the status changes to
            S. S The transaction successfully completed without a denial and
            after any pending statuses. V A successful transaction was fully
            reversed and funds were refunded to the original sender.
        transaction_subject (str): The subject of payment. The payer passes
            this value to the payee. The payer controls this data through the
            interface through which he or she sends the data.
        transaction_note (str): A special note that the payer passes to the
            payee. Might contain special customer requests, such as shipping
            instructions.
        payment_tracking_id (str): The payment tracking ID, which is a unique
            ID that partners specify to either get information about a payment
            or request a refund.
        bank_reference_id (str): The bank reference ID. The bank provides this
            value for an ACH transaction.
        ending_balance (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        available_balance (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        invoice_id (str): The invoice ID that is sent by the merchant with the
            transaction. Note: If an invoice ID was sent with the capture
            request, the value is reported. Otherwise, the invoice ID of the
            authorizing transaction is reported.
        custom_field (str): The merchant-provided custom text. Note: Usually,
            this field includes the unique ID for payments made with MassPay
            type transaction.
        protection_eligibility (str): Indicates whether the transaction is
            eligible for protection. Value is: 01. Eligible. 02. Not eligible
            03. Partially eligible.
        credit_term (str): The credit term. The time span covered by the
            installment payments as expressed in the term length plus the
            length time unit code.
        credit_transactional_fee (Money): The currency and amount for a
            financial transaction, such as a balance or payment due.
        credit_promotional_fee (Money): The currency and amount for a
            financial transaction, such as a balance or payment due.
        annual_percentage_rate (str): The percentage, as a fixed-point, signed
            decimal number. For example, define a 19.99% interest rate as
            `19.99`.
        payment_method_type (str): The payment method that was used for a
            transaction. Value is PUI, installment, or mEFT. Note: Appears
            only for pay upon invoice (PUI), installment, and mEFT
            transactions. Merchants and partners in the EMEA region can use
            this attribute to note transactions that attract turn-over tax.
        instrument_type (str): A high-level classification of the type of
            financial instrument that was used to fund a payment. The pattern
            is not provided because the value is defined by an external party.
            E.g. PAYPAL, CREDIT_CARD, DEBIT_CARD, APPLE_PAY, BANK , VENMO ,Pay
            Upon Invoice, Pay Later  or Alternative Payment Methods (APM).
        instrument_sub_type (str): A finer-grained classification of the
            financial instrument that was used to fund a payment. For example,
            `Visa card` or a `Mastercard` for a credit card, BANKCARD
            ,DISCOVER etc. The pattern is not provided because the value is
            defined by an external party.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "paypal_account_id": 'paypal_account_id',
        "transaction_id": 'transaction_id',
        "paypal_reference_id": 'paypal_reference_id',
        "paypal_reference_id_type": 'paypal_reference_id_type',
        "transaction_event_code": 'transaction_event_code',
        "transaction_initiation_date": 'transaction_initiation_date',
        "transaction_updated_date": 'transaction_updated_date',
        "transaction_amount": 'transaction_amount',
        "fee_amount": 'fee_amount',
        "discount_amount": 'discount_amount',
        "insurance_amount": 'insurance_amount',
        "sales_tax_amount": 'sales_tax_amount',
        "shipping_amount": 'shipping_amount',
        "shipping_discount_amount": 'shipping_discount_amount',
        "shipping_tax_amount": 'shipping_tax_amount',
        "other_amount": 'other_amount',
        "tip_amount": 'tip_amount',
        "transaction_status": 'transaction_status',
        "transaction_subject": 'transaction_subject',
        "transaction_note": 'transaction_note',
        "payment_tracking_id": 'payment_tracking_id',
        "bank_reference_id": 'bank_reference_id',
        "ending_balance": 'ending_balance',
        "available_balance": 'available_balance',
        "invoice_id": 'invoice_id',
        "custom_field": 'custom_field',
        "protection_eligibility": 'protection_eligibility',
        "credit_term": 'credit_term',
        "credit_transactional_fee": 'credit_transactional_fee',
        "credit_promotional_fee": 'credit_promotional_fee',
        "annual_percentage_rate": 'annual_percentage_rate',
        "payment_method_type": 'payment_method_type',
        "instrument_type": 'instrument_type',
        "instrument_sub_type": 'instrument_sub_type'
    }

    _optionals = [
        'paypal_account_id',
        'transaction_id',
        'paypal_reference_id',
        'paypal_reference_id_type',
        'transaction_event_code',
        'transaction_initiation_date',
        'transaction_updated_date',
        'transaction_amount',
        'fee_amount',
        'discount_amount',
        'insurance_amount',
        'sales_tax_amount',
        'shipping_amount',
        'shipping_discount_amount',
        'shipping_tax_amount',
        'other_amount',
        'tip_amount',
        'transaction_status',
        'transaction_subject',
        'transaction_note',
        'payment_tracking_id',
        'bank_reference_id',
        'ending_balance',
        'available_balance',
        'invoice_id',
        'custom_field',
        'protection_eligibility',
        'credit_term',
        'credit_transactional_fee',
        'credit_promotional_fee',
        'annual_percentage_rate',
        'payment_method_type',
        'instrument_type',
        'instrument_sub_type',
    ]

    def __init__(self,
                 paypal_account_id=APIHelper.SKIP,
                 transaction_id=APIHelper.SKIP,
                 paypal_reference_id=APIHelper.SKIP,
                 paypal_reference_id_type=APIHelper.SKIP,
                 transaction_event_code=APIHelper.SKIP,
                 transaction_initiation_date=APIHelper.SKIP,
                 transaction_updated_date=APIHelper.SKIP,
                 transaction_amount=APIHelper.SKIP,
                 fee_amount=APIHelper.SKIP,
                 discount_amount=APIHelper.SKIP,
                 insurance_amount=APIHelper.SKIP,
                 sales_tax_amount=APIHelper.SKIP,
                 shipping_amount=APIHelper.SKIP,
                 shipping_discount_amount=APIHelper.SKIP,
                 shipping_tax_amount=APIHelper.SKIP,
                 other_amount=APIHelper.SKIP,
                 tip_amount=APIHelper.SKIP,
                 transaction_status=APIHelper.SKIP,
                 transaction_subject=APIHelper.SKIP,
                 transaction_note=APIHelper.SKIP,
                 payment_tracking_id=APIHelper.SKIP,
                 bank_reference_id=APIHelper.SKIP,
                 ending_balance=APIHelper.SKIP,
                 available_balance=APIHelper.SKIP,
                 invoice_id=APIHelper.SKIP,
                 custom_field=APIHelper.SKIP,
                 protection_eligibility=APIHelper.SKIP,
                 credit_term=APIHelper.SKIP,
                 credit_transactional_fee=APIHelper.SKIP,
                 credit_promotional_fee=APIHelper.SKIP,
                 annual_percentage_rate=APIHelper.SKIP,
                 payment_method_type=APIHelper.SKIP,
                 instrument_type=APIHelper.SKIP,
                 instrument_sub_type=APIHelper.SKIP):
        """Constructor for the TransactionInformation class"""

        # Initialize members of the class
        if paypal_account_id is not APIHelper.SKIP:
            self.paypal_account_id = paypal_account_id 
        if transaction_id is not APIHelper.SKIP:
            self.transaction_id = transaction_id 
        if paypal_reference_id is not APIHelper.SKIP:
            self.paypal_reference_id = paypal_reference_id 
        if paypal_reference_id_type is not APIHelper.SKIP:
            self.paypal_reference_id_type = paypal_reference_id_type 
        if transaction_event_code is not APIHelper.SKIP:
            self.transaction_event_code = transaction_event_code 
        if transaction_initiation_date is not APIHelper.SKIP:
            self.transaction_initiation_date = transaction_initiation_date 
        if transaction_updated_date is not APIHelper.SKIP:
            self.transaction_updated_date = transaction_updated_date 
        if transaction_amount is not APIHelper.SKIP:
            self.transaction_amount = transaction_amount 
        if fee_amount is not APIHelper.SKIP:
            self.fee_amount = fee_amount 
        if discount_amount is not APIHelper.SKIP:
            self.discount_amount = discount_amount 
        if insurance_amount is not APIHelper.SKIP:
            self.insurance_amount = insurance_amount 
        if sales_tax_amount is not APIHelper.SKIP:
            self.sales_tax_amount = sales_tax_amount 
        if shipping_amount is not APIHelper.SKIP:
            self.shipping_amount = shipping_amount 
        if shipping_discount_amount is not APIHelper.SKIP:
            self.shipping_discount_amount = shipping_discount_amount 
        if shipping_tax_amount is not APIHelper.SKIP:
            self.shipping_tax_amount = shipping_tax_amount 
        if other_amount is not APIHelper.SKIP:
            self.other_amount = other_amount 
        if tip_amount is not APIHelper.SKIP:
            self.tip_amount = tip_amount 
        if transaction_status is not APIHelper.SKIP:
            self.transaction_status = transaction_status 
        if transaction_subject is not APIHelper.SKIP:
            self.transaction_subject = transaction_subject 
        if transaction_note is not APIHelper.SKIP:
            self.transaction_note = transaction_note 
        if payment_tracking_id is not APIHelper.SKIP:
            self.payment_tracking_id = payment_tracking_id 
        if bank_reference_id is not APIHelper.SKIP:
            self.bank_reference_id = bank_reference_id 
        if ending_balance is not APIHelper.SKIP:
            self.ending_balance = ending_balance 
        if available_balance is not APIHelper.SKIP:
            self.available_balance = available_balance 
        if invoice_id is not APIHelper.SKIP:
            self.invoice_id = invoice_id 
        if custom_field is not APIHelper.SKIP:
            self.custom_field = custom_field 
        if protection_eligibility is not APIHelper.SKIP:
            self.protection_eligibility = protection_eligibility 
        if credit_term is not APIHelper.SKIP:
            self.credit_term = credit_term 
        if credit_transactional_fee is not APIHelper.SKIP:
            self.credit_transactional_fee = credit_transactional_fee 
        if credit_promotional_fee is not APIHelper.SKIP:
            self.credit_promotional_fee = credit_promotional_fee 
        if annual_percentage_rate is not APIHelper.SKIP:
            self.annual_percentage_rate = annual_percentage_rate 
        if payment_method_type is not APIHelper.SKIP:
            self.payment_method_type = payment_method_type 
        if instrument_type is not APIHelper.SKIP:
            self.instrument_type = instrument_type 
        if instrument_sub_type is not APIHelper.SKIP:
            self.instrument_sub_type = instrument_sub_type 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        paypal_account_id = dictionary.get("paypal_account_id") if dictionary.get("paypal_account_id") else APIHelper.SKIP
        transaction_id = dictionary.get("transaction_id") if dictionary.get("transaction_id") else APIHelper.SKIP
        paypal_reference_id = dictionary.get("paypal_reference_id") if dictionary.get("paypal_reference_id") else APIHelper.SKIP
        paypal_reference_id_type = dictionary.get("paypal_reference_id_type") if dictionary.get("paypal_reference_id_type") else APIHelper.SKIP
        transaction_event_code = dictionary.get("transaction_event_code") if dictionary.get("transaction_event_code") else APIHelper.SKIP
        transaction_initiation_date = dictionary.get("transaction_initiation_date") if dictionary.get("transaction_initiation_date") else APIHelper.SKIP
        transaction_updated_date = dictionary.get("transaction_updated_date") if dictionary.get("transaction_updated_date") else APIHelper.SKIP
        transaction_amount = Money.from_dictionary(dictionary.get('transaction_amount')) if 'transaction_amount' in dictionary.keys() else APIHelper.SKIP
        fee_amount = Money.from_dictionary(dictionary.get('fee_amount')) if 'fee_amount' in dictionary.keys() else APIHelper.SKIP
        discount_amount = Money.from_dictionary(dictionary.get('discount_amount')) if 'discount_amount' in dictionary.keys() else APIHelper.SKIP
        insurance_amount = Money.from_dictionary(dictionary.get('insurance_amount')) if 'insurance_amount' in dictionary.keys() else APIHelper.SKIP
        sales_tax_amount = Money.from_dictionary(dictionary.get('sales_tax_amount')) if 'sales_tax_amount' in dictionary.keys() else APIHelper.SKIP
        shipping_amount = Money.from_dictionary(dictionary.get('shipping_amount')) if 'shipping_amount' in dictionary.keys() else APIHelper.SKIP
        shipping_discount_amount = Money.from_dictionary(dictionary.get('shipping_discount_amount')) if 'shipping_discount_amount' in dictionary.keys() else APIHelper.SKIP
        shipping_tax_amount = Money.from_dictionary(dictionary.get('shipping_tax_amount')) if 'shipping_tax_amount' in dictionary.keys() else APIHelper.SKIP
        other_amount = Money.from_dictionary(dictionary.get('other_amount')) if 'other_amount' in dictionary.keys() else APIHelper.SKIP
        tip_amount = Money.from_dictionary(dictionary.get('tip_amount')) if 'tip_amount' in dictionary.keys() else APIHelper.SKIP
        transaction_status = dictionary.get("transaction_status") if dictionary.get("transaction_status") else APIHelper.SKIP
        transaction_subject = dictionary.get("transaction_subject") if dictionary.get("transaction_subject") else APIHelper.SKIP
        transaction_note = dictionary.get("transaction_note") if dictionary.get("transaction_note") else APIHelper.SKIP
        payment_tracking_id = dictionary.get("payment_tracking_id") if dictionary.get("payment_tracking_id") else APIHelper.SKIP
        bank_reference_id = dictionary.get("bank_reference_id") if dictionary.get("bank_reference_id") else APIHelper.SKIP
        ending_balance = Money.from_dictionary(dictionary.get('ending_balance')) if 'ending_balance' in dictionary.keys() else APIHelper.SKIP
        available_balance = Money.from_dictionary(dictionary.get('available_balance')) if 'available_balance' in dictionary.keys() else APIHelper.SKIP
        invoice_id = dictionary.get("invoice_id") if dictionary.get("invoice_id") else APIHelper.SKIP
        custom_field = dictionary.get("custom_field") if dictionary.get("custom_field") else APIHelper.SKIP
        protection_eligibility = dictionary.get("protection_eligibility") if dictionary.get("protection_eligibility") else APIHelper.SKIP
        credit_term = dictionary.get("credit_term") if dictionary.get("credit_term") else APIHelper.SKIP
        credit_transactional_fee = Money.from_dictionary(dictionary.get('credit_transactional_fee')) if 'credit_transactional_fee' in dictionary.keys() else APIHelper.SKIP
        credit_promotional_fee = Money.from_dictionary(dictionary.get('credit_promotional_fee')) if 'credit_promotional_fee' in dictionary.keys() else APIHelper.SKIP
        annual_percentage_rate = dictionary.get("annual_percentage_rate") if dictionary.get("annual_percentage_rate") else APIHelper.SKIP
        payment_method_type = dictionary.get("payment_method_type") if dictionary.get("payment_method_type") else APIHelper.SKIP
        instrument_type = dictionary.get("instrument_type") if dictionary.get("instrument_type") else APIHelper.SKIP
        instrument_sub_type = dictionary.get("instrument_sub_type") if dictionary.get("instrument_sub_type") else APIHelper.SKIP
        # Return an object of this model
        return cls(paypal_account_id,
                   transaction_id,
                   paypal_reference_id,
                   paypal_reference_id_type,
                   transaction_event_code,
                   transaction_initiation_date,
                   transaction_updated_date,
                   transaction_amount,
                   fee_amount,
                   discount_amount,
                   insurance_amount,
                   sales_tax_amount,
                   shipping_amount,
                   shipping_discount_amount,
                   shipping_tax_amount,
                   other_amount,
                   tip_amount,
                   transaction_status,
                   transaction_subject,
                   transaction_note,
                   payment_tracking_id,
                   bank_reference_id,
                   ending_balance,
                   available_balance,
                   invoice_id,
                   custom_field,
                   protection_eligibility,
                   credit_term,
                   credit_transactional_fee,
                   credit_promotional_fee,
                   annual_percentage_rate,
                   payment_method_type,
                   instrument_type,
                   instrument_sub_type)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'paypal_account_id={(self.paypal_account_id if hasattr(self, "paypal_account_id") else None)!r}, '
                f'transaction_id={(self.transaction_id if hasattr(self, "transaction_id") else None)!r}, '
                f'paypal_reference_id={(self.paypal_reference_id if hasattr(self, "paypal_reference_id") else None)!r}, '
                f'paypal_reference_id_type={(self.paypal_reference_id_type if hasattr(self, "paypal_reference_id_type") else None)!r}, '
                f'transaction_event_code={(self.transaction_event_code if hasattr(self, "transaction_event_code") else None)!r}, '
                f'transaction_initiation_date={(self.transaction_initiation_date if hasattr(self, "transaction_initiation_date") else None)!r}, '
                f'transaction_updated_date={(self.transaction_updated_date if hasattr(self, "transaction_updated_date") else None)!r}, '
                f'transaction_amount={(self.transaction_amount if hasattr(self, "transaction_amount") else None)!r}, '
                f'fee_amount={(self.fee_amount if hasattr(self, "fee_amount") else None)!r}, '
                f'discount_amount={(self.discount_amount if hasattr(self, "discount_amount") else None)!r}, '
                f'insurance_amount={(self.insurance_amount if hasattr(self, "insurance_amount") else None)!r}, '
                f'sales_tax_amount={(self.sales_tax_amount if hasattr(self, "sales_tax_amount") else None)!r}, '
                f'shipping_amount={(self.shipping_amount if hasattr(self, "shipping_amount") else None)!r}, '
                f'shipping_discount_amount={(self.shipping_discount_amount if hasattr(self, "shipping_discount_amount") else None)!r}, '
                f'shipping_tax_amount={(self.shipping_tax_amount if hasattr(self, "shipping_tax_amount") else None)!r}, '
                f'other_amount={(self.other_amount if hasattr(self, "other_amount") else None)!r}, '
                f'tip_amount={(self.tip_amount if hasattr(self, "tip_amount") else None)!r}, '
                f'transaction_status={(self.transaction_status if hasattr(self, "transaction_status") else None)!r}, '
                f'transaction_subject={(self.transaction_subject if hasattr(self, "transaction_subject") else None)!r}, '
                f'transaction_note={(self.transaction_note if hasattr(self, "transaction_note") else None)!r}, '
                f'payment_tracking_id={(self.payment_tracking_id if hasattr(self, "payment_tracking_id") else None)!r}, '
                f'bank_reference_id={(self.bank_reference_id if hasattr(self, "bank_reference_id") else None)!r}, '
                f'ending_balance={(self.ending_balance if hasattr(self, "ending_balance") else None)!r}, '
                f'available_balance={(self.available_balance if hasattr(self, "available_balance") else None)!r}, '
                f'invoice_id={(self.invoice_id if hasattr(self, "invoice_id") else None)!r}, '
                f'custom_field={(self.custom_field if hasattr(self, "custom_field") else None)!r}, '
                f'protection_eligibility={(self.protection_eligibility if hasattr(self, "protection_eligibility") else None)!r}, '
                f'credit_term={(self.credit_term if hasattr(self, "credit_term") else None)!r}, '
                f'credit_transactional_fee={(self.credit_transactional_fee if hasattr(self, "credit_transactional_fee") else None)!r}, '
                f'credit_promotional_fee={(self.credit_promotional_fee if hasattr(self, "credit_promotional_fee") else None)!r}, '
                f'annual_percentage_rate={(self.annual_percentage_rate if hasattr(self, "annual_percentage_rate") else None)!r}, '
                f'payment_method_type={(self.payment_method_type if hasattr(self, "payment_method_type") else None)!r}, '
                f'instrument_type={(self.instrument_type if hasattr(self, "instrument_type") else None)!r}, '
                f'instrument_sub_type={(self.instrument_sub_type if hasattr(self, "instrument_sub_type") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'paypal_account_id={(self.paypal_account_id if hasattr(self, "paypal_account_id") else None)!s}, '
                f'transaction_id={(self.transaction_id if hasattr(self, "transaction_id") else None)!s}, '
                f'paypal_reference_id={(self.paypal_reference_id if hasattr(self, "paypal_reference_id") else None)!s}, '
                f'paypal_reference_id_type={(self.paypal_reference_id_type if hasattr(self, "paypal_reference_id_type") else None)!s}, '
                f'transaction_event_code={(self.transaction_event_code if hasattr(self, "transaction_event_code") else None)!s}, '
                f'transaction_initiation_date={(self.transaction_initiation_date if hasattr(self, "transaction_initiation_date") else None)!s}, '
                f'transaction_updated_date={(self.transaction_updated_date if hasattr(self, "transaction_updated_date") else None)!s}, '
                f'transaction_amount={(self.transaction_amount if hasattr(self, "transaction_amount") else None)!s}, '
                f'fee_amount={(self.fee_amount if hasattr(self, "fee_amount") else None)!s}, '
                f'discount_amount={(self.discount_amount if hasattr(self, "discount_amount") else None)!s}, '
                f'insurance_amount={(self.insurance_amount if hasattr(self, "insurance_amount") else None)!s}, '
                f'sales_tax_amount={(self.sales_tax_amount if hasattr(self, "sales_tax_amount") else None)!s}, '
                f'shipping_amount={(self.shipping_amount if hasattr(self, "shipping_amount") else None)!s}, '
                f'shipping_discount_amount={(self.shipping_discount_amount if hasattr(self, "shipping_discount_amount") else None)!s}, '
                f'shipping_tax_amount={(self.shipping_tax_amount if hasattr(self, "shipping_tax_amount") else None)!s}, '
                f'other_amount={(self.other_amount if hasattr(self, "other_amount") else None)!s}, '
                f'tip_amount={(self.tip_amount if hasattr(self, "tip_amount") else None)!s}, '
                f'transaction_status={(self.transaction_status if hasattr(self, "transaction_status") else None)!s}, '
                f'transaction_subject={(self.transaction_subject if hasattr(self, "transaction_subject") else None)!s}, '
                f'transaction_note={(self.transaction_note if hasattr(self, "transaction_note") else None)!s}, '
                f'payment_tracking_id={(self.payment_tracking_id if hasattr(self, "payment_tracking_id") else None)!s}, '
                f'bank_reference_id={(self.bank_reference_id if hasattr(self, "bank_reference_id") else None)!s}, '
                f'ending_balance={(self.ending_balance if hasattr(self, "ending_balance") else None)!s}, '
                f'available_balance={(self.available_balance if hasattr(self, "available_balance") else None)!s}, '
                f'invoice_id={(self.invoice_id if hasattr(self, "invoice_id") else None)!s}, '
                f'custom_field={(self.custom_field if hasattr(self, "custom_field") else None)!s}, '
                f'protection_eligibility={(self.protection_eligibility if hasattr(self, "protection_eligibility") else None)!s}, '
                f'credit_term={(self.credit_term if hasattr(self, "credit_term") else None)!s}, '
                f'credit_transactional_fee={(self.credit_transactional_fee if hasattr(self, "credit_transactional_fee") else None)!s}, '
                f'credit_promotional_fee={(self.credit_promotional_fee if hasattr(self, "credit_promotional_fee") else None)!s}, '
                f'annual_percentage_rate={(self.annual_percentage_rate if hasattr(self, "annual_percentage_rate") else None)!s}, '
                f'payment_method_type={(self.payment_method_type if hasattr(self, "payment_method_type") else None)!s}, '
                f'instrument_type={(self.instrument_type if hasattr(self, "instrument_type") else None)!s}, '
                f'instrument_sub_type={(self.instrument_sub_type if hasattr(self, "instrument_sub_type") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.address import Address
from paypalserversdk.models.name import Name
from paypalserversdk.models.phone_with_type import PhoneWithType
from paypalserversdk.models.tax_info import TaxInfo


class Payer(object):

    """Implementation of the 'Payer' model.

    The customer who approves and pays for the order. The customer is also
    known as the payer.

    Attributes:
        email_address (str): The internationalized email address. Note: Up to
            64 characters are allowed before and 255 characters are allowed
            after the @ sign. However, the generally accepted maximum length
            for an email address is 254 characters. The pattern verifies that
            an unquoted @ sign exists.
        payer_id (str): The account identifier for a PayPal account.
        name (Name): The name of the party.
        phone (PhoneWithType): The phone information.
        birth_date (str): The stand-alone date, in [Internet date and time
            format](https://tools.ietf.org/html/rfc3339#section-5.6). To
            represent special legal values, such as a date of birth, you
            should use dates with no associated time or time-zone data.
            Whenever possible, use the standard `date_time` type. This regular
            expression does not validate all dates. For example, February 31
            is valid and nothing is known about leap years.
        tax_info (TaxInfo): The tax ID of the customer. The customer is also
            known as the payer. Both `tax_id` and `tax_id_type` are required.
        address (Address): The portable international postal address. Maps to
            [AddressValidationMetadata](https://github.com/googlei18n/libaddres
            sinput/wiki/AddressValidationMetadata) and HTML 5.1 [Autofilling
            form controls: the autocomplete
            attribute](https://www.w3.org/TR/html51/sec-forms.html#autofilling-
            form-controls-the-autocomplete-attribute).

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "email_address": 'email_address',
        "payer_id": 'payer_id',
        "name": 'name',
        "phone": 'phone',
        "birth_date": 'birth_date',
        "tax_info": 'tax_info',
        "address": 'address'
    }

    _optionals = [
        'email_address',
        'payer_id',
        'name',
        'phone',
        'birth_date',
        'tax_info',
        'address',
    ]

    def __init__(self,
                 email_address=APIHelper.SKIP,
                 payer_id=APIHelper.SKIP,
                 name=APIHelper.SKIP,
                 phone=APIHelper.SKIP,
                 birth_date=APIHelper.SKIP,
                 tax_info=APIHelper.SKIP,
                 address=APIHelper.SKIP):
        """Constructor for the Payer class"""

        # Initialize members of the class
        if email_address is not APIHelper.SKIP:
            self.email_address = email_address 
        if payer_id is not APIHelper.SKIP:
            self.payer_id = payer_id 
        if name is not APIHelper.SKIP:
            self.name = name 
        if phone is not APIHelper.SKIP:
            self.phone = phone 
        if birth_date is not APIHelper.SKIP:
            self.birth_date = birth_date 
        if tax_info is not APIHelper.SKIP:
            self.tax_info = tax_info 
        if address is not APIHelper.SKIP:
            self.address = address 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        email_address = dictionary.get("email_address") if dictionary.get("email_address") else APIHelper.SKIP
        payer_id = dictionary.get("payer_id") if dictionary.get("payer_id") else APIHelper.SKIP
        name = Name.from_dictionary(dictionary.get('name')) if 'name' in dictionary.keys() else APIHelper.SKIP
        phone = PhoneWithType.from_dictionary(dictionary.get('phone')) if 'phone' in dictionary.keys() else APIHelper.SKIP
        birth_date = dictionary.get("birth_date") if dictionary.get("birth_date") else APIHelper.SKIP
        tax_info = TaxInfo.from_dictionary(dictionary.get('tax_info')) if 'tax_info' in dictionary.keys() else APIHelper.SKIP
        address = Address.from_dictionary(dictionary.get('address')) if 'address' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(email_address,
                   payer_id,
                   name,
                   phone,
                   birth_date,
                   tax_info,
                   address)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'email_address={(self.email_address if hasattr(self, "email_address") else None)!r}, '
                f'payer_id={(self.payer_id if hasattr(self, "payer_id") else None)!r}, '
                f'name={(self.name if hasattr(self, "name") else None)!r}, '
                f'phone={(self.phone if hasattr(self, "phone") else None)!r}, '
                f'birth_date={(self.birth_date if hasattr(self, "birth_date") else None)!r}, '
                f'tax_info={(self.tax_info if hasattr(self, "tax_info") else None)!r}, '
                f'address={(self.address if hasattr(self, "address") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'email_address={(self.email_address if hasattr(self, "email_address") else None)!s}, '
                f'payer_id={(self.payer_id if hasattr(self, "payer_id") else None)!s}, '
                f'name={(self.name if hasattr(self, "name") else None)!s}, '
                f'phone={(self.phone if hasattr(self, "phone") else None)!s}, '
                f'birth_date={(self.birth_date if hasattr(self, "birth_date") else None)!s}, '
                f'tax_info={(self.tax_info if hasattr(self, "tax_info") else None)!s}, '
                f'address={(self.address if hasattr(self, "address") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class PaypalPaymentTokenUsageType(object):

    """Implementation of the 'PayPal Payment Token Usage Type' enum.

    The usage type associated with the PayPal payment token., The usage type
    associated with a digital wallet payment token.

    Attributes:
        MERCHANT: The PayPal Payment Token will be used for future transaction
            directly with a merchant.
        PLATFORM: The PayPal Payment Token will be used for future transaction
            on a platform. A platform is typically a marketplace or a channel
            that a payer can purchase goods and services from multiple
            merchants.

    """
    MERCHANT = 'MERCHANT'

    PLATFORM = 'PLATFORM'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.money import Money


class LastPaymentDetails(object):

    """Implementation of the 'Last Payment Details' model.

    Attributes:
        amount (Money): The currency and amount for a financial transaction,
            such as a balance or payment due.
        time (str): The date and time, in [Internet date and time
            format](https://tools.ietf.org/html/rfc3339#section-5.6). Seconds
            are required while fractional seconds are optional. Note: The
            regular expression provides guidance but does not reject all
            invalid dates.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "amount": 'amount',
        "time": 'time'
    }

    _optionals = [
        'amount',
        'time',
    ]

    def __init__(self,
                 amount=APIHelper.SKIP,
                 time=APIHelper.SKIP):
        """Constructor for the LastPaymentDetails class"""

        # Initialize members of the class
        if amount is not APIHelper.SKIP:
            self.amount = amount 
        if time is not APIHelper.SKIP:
            self.time = time 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        amount = Money.from_dictionary(dictionary.get('amount')) if 'amount' in dictionary.keys() else APIHelper.SKIP
        time = dictionary.get("time") if dictionary.get("time") else APIHelper.SKIP
        # Return an object of this model
        return cls(amount,
                   time)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'amount={(self.amount if hasattr(self, "amount") else None)!r}, '
                f'time={(self.time if hasattr(self, "time") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'amount={(self.amount if hasattr(self, "amount") else None)!s}, '
                f'time={(self.time if hasattr(self, "time") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.billing_plan import BillingPlan
from paypalserversdk.models.link_description import LinkDescription


class PlanCollection(object):

    """Implementation of the 'Plan Collection' model.

    The list of plans with details.

    Attributes:
        plans (List[BillingPlan]): An array of plans.
        total_items (int): The total number of items.
        total_pages (int): The total number of pages.
        links (List[LinkDescription]): An array of request-related [HATEOAS
            links](/docs/api/reference/api-responses/#hateoas-links).

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "plans": 'plans',
        "total_items": 'total_items',
        "total_pages": 'total_pages',
        "links": 'links'
    }

    _optionals = [
        'plans',
        'total_items',
        'total_pages',
        'links',
    ]

    def __init__(self,
                 plans=APIHelper.SKIP,
                 total_items=APIHelper.SKIP,
                 total_pages=APIHelper.SKIP,
                 links=APIHelper.SKIP):
        """Constructor for the PlanCollection class"""

        # Initialize members of the class
        if plans is not APIHelper.SKIP:
            self.plans = plans 
        if total_items is not APIHelper.SKIP:
            self.total_items = total_items 
        if total_pages is not APIHelper.SKIP:
            self.total_pages = total_pages 
        if links is not APIHelper.SKIP:
            self.links = links 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        plans = None
        if dictionary.get('plans') is not None:
            plans = [BillingPlan.from_dictionary(x) for x in dictionary.get('plans')]
        else:
            plans = APIHelper.SKIP
        total_items = dictionary.get("total_items") if dictionary.get("total_items") else APIHelper.SKIP
        total_pages = dictionary.get("total_pages") if dictionary.get("total_pages") else APIHelper.SKIP
        links = None
        if dictionary.get('links') is not None:
            links = [LinkDescription.from_dictionary(x) for x in dictionary.get('links')]
        else:
            links = APIHelper.SKIP
        # Return an object of this model
        return cls(plans,
                   total_items,
                   total_pages,
                   links)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'plans={(self.plans if hasattr(self, "plans") else None)!r}, '
                f'total_items={(self.total_items if hasattr(self, "total_items") else None)!r}, '
                f'total_pages={(self.total_pages if hasattr(self, "total_pages") else None)!r}, '
                f'links={(self.links if hasattr(self, "links") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'plans={(self.plans if hasattr(self, "plans") else None)!s}, '
                f'total_items={(self.total_items if hasattr(self, "total_items") else None)!s}, '
                f'total_pages={(self.total_pages if hasattr(self, "total_pages") else None)!s}, '
                f'links={(self.links if hasattr(self, "links") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from paypalserversdk.api_helper import APIHelper
from paypalserversdk.configuration import Server
from paypalserversdk.http.api_response import ApiResponse
from paypalserversdk.controllers.base_controller import BaseController
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter
from paypalserversdk.http.http_method_enum import HttpMethodEnum
from apimatic_core.authentication.multiple.single_auth import Single
from paypalserversdk.models.search_response import SearchResponse
from paypalserversdk.models.balances_response import BalancesResponse
from paypalserversdk.exceptions.search_error_exception import SearchErrorException
from paypalserversdk.exceptions.default_error_exception import DefaultErrorException


class TransactionSearchController(BaseController):

    """A Controller to access Endpoints in the paypalserversdk API."""
    def __init__(self, config):
        super(TransactionSearchController, self).__init__(config)

    def search_transactions(self,
                            options=dict()):
        """Does a GET request to /v1/reporting/transactions.

        Lists transactions. Specify one or more query parameters to filter the
        transaction that appear in the response. Notes: If you specify one or
        more optional query parameters, the ending_balance response field is
        empty. It takes a maximum of three hours for executed transactions to
        appear in the list transactions call. This call lists transaction for
        the previous three years.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    start_date -- str -- Filters the transactions in the
                        response by a start date and time, in [Internet date
                        and time
                        format](https://tools.ietf.org/html/rfc3339#section-5.6
                        ). Seconds are required. Fractional seconds are
                        optional.
                    end_date -- str -- Filters the transactions in the
                        response by an end date and time, in [Internet date
                        and time
                        format](https://tools.ietf.org/html/rfc3339#section-5.6
                        ). Seconds are required. Fractional seconds are
                        optional. The maximum supported range is 31 days.
                    transaction_id -- str -- Filters the transactions in the
                        response by a PayPal transaction ID. A valid
                        transaction ID is 17 characters long, except for an
                        order ID, which is 19 characters long. Note: A
                        transaction ID is not unique in the reporting system.
                        The response can list two transactions with the same
                        ID. One transaction can be balance affecting while the
                        other is non-balance affecting.
                    transaction_type -- str -- Filters the transactions in the
                        response by a PayPal transaction event code. See
                        [Transaction event
                        codes](/docs/integration/direct/transaction-search/tran
                        saction-event-codes/).
                    transaction_status -- str -- Filters the transactions in
                        the response by a PayPal transaction status code.
                        Value is: Status code Description D PayPal or merchant
                        rules denied the transaction. P The transaction is
                        pending. The transaction was created but waits for
                        another payment process to complete, such as an ACH
                        transaction, before the status changes to S. S The
                        transaction successfully completed without a denial
                        and after any pending statuses. V A successful
                        transaction was reversed and funds were refunded to
                        the original sender.
                    transaction_amount -- str -- Filters the transactions in
                        the response by a gross transaction amount range.
                        Specify the range as ` TO `, where ` ` is the lower
                        limit of the gross PayPal transaction amount and ` `
                        is the upper limit of the gross transaction amount.
                        Specify the amounts in lower denominations. For
                        example, to search for transactions from $5.00 to
                        $10.05, specify `[500 TO 1005]`. Note:The values must
                        be URL encoded.
                    transaction_currency -- str -- Filters the transactions in
                        the response by a [three-character ISO-4217 currency
                        code](/api/rest/reference/currency-codes/) for the
                        PayPal transaction currency.
                    payment_instrument_type -- str -- Filters the transactions
                        in the response by a payment instrument type. Value is
                        either: CREDITCARD. Returns a direct credit card
                        transaction with a corresponding value. DEBITCARD.
                        Returns a debit card transaction with a corresponding
                        value. If you omit this parameter, the API does not
                        apply this filter.
                    store_id -- str -- Filters the transactions in the
                        response by a store ID.
                    terminal_id -- str -- Filters the transactions in the
                        response by a terminal ID.
                    fields -- str -- Indicates which fields appear in the
                        response. Value is a single field or a comma-separated
                        list of fields. The transaction_info value returns
                        only the transaction details in the response. To
                        include all fields in the response, specify
                        fields=all. Valid fields are: transaction_info. The
                        transaction information. Includes the ID of the PayPal
                        account of the payee, the PayPal-generated transaction
                        ID, the PayPal-generated base ID, the PayPal reference
                        ID type, the transaction event code, the date and time
                        when the transaction was initiated and was last
                        updated, the transaction amounts including the PayPal
                        fee, any discounts, insurance, the transaction status,
                        and other information about the transaction.
                        payer_info. The payer information. Includes the PayPal
                        customer account ID and the payer's email address,
                        primary phone number, name, country code, address, and
                        whether the payer is verified or unverified.
                        shipping_info. The shipping information. Includes the
                        recipient's name, the shipping method for this order,
                        the shipping address for this order, and the secondary
                        address associated with this order. auction_info. The
                        auction information. Includes the name of the auction
                        site, the auction site URL, the ID of the customer who
                        makes the purchase in the auction, and the date and
                        time when the auction closes. cart_info. The cart
                        information. Includes an array of item details,
                        whether the item amount or the shipping amount already
                        includes tax, and the ID of the invoice for
                        PayPal-generated invoices. incentive_info. An array of
                        incentive detail objects. Each object includes the
                        incentive, such as a special offer or coupon, the
                        incentive amount, and the incentive program code that
                        identifies a merchant loyalty or incentive program.
                        store_info. The store information. Includes the ID of
                        the merchant store and the terminal ID for the
                        checkout stand in the merchant store.
                    balance_affecting_records_only -- str -- Indicates whether
                        the response includes only balance-impacting
                        transactions or all transactions. Value is either: Y.
                        The default. The response includes only balance
                        transactions. N. The response includes all
                        transactions.
                    page_size -- int -- The number of items to return in the
                        response. So, the combination of `page=1` and
                        `page_size=20` returns the first 20 items. The
                        combination of `page=2` and `page_size=20` returns the
                        next 20 items.
                    page -- int -- The zero-relative start index of the entire
                        list of items that are returned in the response. So,
                        the combination of `page=1` and `page_size=20` returns
                        the first 20 items.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP `200 OK` status code and a
                JSON response body that lists transactions .

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v1/reporting/transactions')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('start_date')
                         .value(options.get('start_date', None)))
            .query_param(Parameter()
                         .key('end_date')
                         .value(options.get('end_date', None)))
            .query_param(Parameter()
                         .key('transaction_id')
                         .value(options.get('transaction_id', None)))
            .query_param(Parameter()
                         .key('transaction_type')
                         .value(options.get('transaction_type', None)))
            .query_param(Parameter()
                         .key('transaction_status')
                         .value(options.get('transaction_status', None)))
            .query_param(Parameter()
                         .key('transaction_amount')
                         .value(options.get('transaction_amount', None)))
            .query_param(Parameter()
                         .key('transaction_currency')
                         .value(options.get('transaction_currency', None)))
            .query_param(Parameter()
                         .key('payment_instrument_type')
                         .value(options.get('payment_instrument_type', None)))
            .query_param(Parameter()
                         .key('store_id')
                         .value(options.get('store_id', None)))
            .query_param(Parameter()
                         .key('terminal_id')
                         .value(options.get('terminal_id', None)))
            .query_param(Parameter()
                         .key('fields')
                         .value(options.get('fields', None)))
            .query_param(Parameter()
                         .key('balance_affecting_records_only')
                         .value(options.get('balance_affecting_records_only', None)))
            .query_param(Parameter()
                         .key('page_size')
                         .value(options.get('page_size', None)))
            .query_param(Parameter()
                         .key('page')
                         .value(options.get('page', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(SearchResponse.from_dictionary)
            .is_api_response(True)
            .local_error('default', 'The error response.', SearchErrorException)
        ).execute()

    def search_balances(self,
                        options=dict()):
        """Does a GET request to /v1/reporting/balances.

        List all balances. Specify date time to list balances for that time
        that appear in the response. Notes: It takes a maximum of three hours
        for balances to appear in the list balances call. This call lists
        balances upto the previous three years.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    as_of_time -- str -- List balances in the response at the
                        date time provided, will return the last refreshed
                        balance in the system when not provided.
                    currency_code -- str -- Filters the transactions in the
                        response by a [three-character ISO-4217 currency
                        code](/api/rest/reference/currency-codes/) for the
                        PayPal transaction currency.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP `200 OK` status code and a
                JSON response body that lists balances .

        Raises:
            ApiException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v1/reporting/balances')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('as_of_time')
                         .value(options.get('as_of_time', None)))
            .query_param(Parameter()
                         .key('currency_code')
                         .value(options.get('currency_code', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(BalancesResponse.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'The request is not well-formed, is syntactically incorrect, or violates schema.', DefaultErrorException)
            .local_error('403', 'Authorization failed due to insufficient permissions.', DefaultErrorException)
            .local_error('500', 'An internal server error occurred.', DefaultErrorException)
            .local_error('default', 'The error response.', DefaultErrorException)
        ).execute()



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class EnrollmentStatus(object):

    """Implementation of the 'Enrollment Status' enum.

    Status of Authentication eligibility.

    Attributes:
        ENROLLED: Yes. The bank is participating in 3-D Secure protocol and
            will return the ACSUrl.
        NOTENROLLED: No. The bank is not participating in 3-D Secure protocol.
        UNAVAILABLE: Unavailable. The DS or ACS is not available for
            authentication at the time of the request.
        BYPASS: Bypass. The merchant authentication rule is triggered to
            bypass authentication.

    """
    ENROLLED = 'Y'

    NOTENROLLED = 'N'

    UNAVAILABLE = 'U'

    BYPASS = 'B'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper


class CycleExecution(object):

    """Implementation of the 'Cycle Execution' model.

    The regular and trial execution details for a billing cycle.

    Attributes:
        tenure_type (TenureType): The type of the billing cycle.
        sequence (int): The order in which to run this cycle among other
            billing cycles.
        cycles_completed (int): The number of billing cycles that have
            completed.
        cycles_remaining (int): For a finite billing cycle, cycles_remaining
            is the number of remaining cycles. For an infinite billing cycle,
            cycles_remaining is set as 0.
        current_pricing_scheme_version (int): The active pricing scheme
            version for the billing cycle.
        total_cycles (int): The number of times this billing cycle gets
            executed. Trial billing cycles can only be executed a finite
            number of times (value between 1 and 999 for total_cycles).
            Regular billing cycles can be executed infinite times (value of 0
            for total_cycles) or a finite number of times (value between 1 and
            999 for total_cycles).

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "tenure_type": 'tenure_type',
        "sequence": 'sequence',
        "cycles_completed": 'cycles_completed',
        "cycles_remaining": 'cycles_remaining',
        "current_pricing_scheme_version": 'current_pricing_scheme_version',
        "total_cycles": 'total_cycles'
    }

    _optionals = [
        'cycles_remaining',
        'current_pricing_scheme_version',
        'total_cycles',
    ]

    def __init__(self,
                 tenure_type=None,
                 sequence=None,
                 cycles_completed=None,
                 cycles_remaining=APIHelper.SKIP,
                 current_pricing_scheme_version=APIHelper.SKIP,
                 total_cycles=APIHelper.SKIP):
        """Constructor for the CycleExecution class"""

        # Initialize members of the class
        self.tenure_type = tenure_type 
        self.sequence = sequence 
        self.cycles_completed = cycles_completed 
        if cycles_remaining is not APIHelper.SKIP:
            self.cycles_remaining = cycles_remaining 
        if current_pricing_scheme_version is not APIHelper.SKIP:
            self.current_pricing_scheme_version = current_pricing_scheme_version 
        if total_cycles is not APIHelper.SKIP:
            self.total_cycles = total_cycles 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        tenure_type = dictionary.get("tenure_type") if dictionary.get("tenure_type") else None
        sequence = dictionary.get("sequence") if dictionary.get("sequence") else None
        cycles_completed = dictionary.get("cycles_completed") if dictionary.get("cycles_completed") else None
        cycles_remaining = dictionary.get("cycles_remaining") if dictionary.get("cycles_remaining") else APIHelper.SKIP
        current_pricing_scheme_version = dictionary.get("current_pricing_scheme_version") if dictionary.get("current_pricing_scheme_version") else APIHelper.SKIP
        total_cycles = dictionary.get("total_cycles") if dictionary.get("total_cycles") else APIHelper.SKIP
        # Return an object of this model
        return cls(tenure_type,
                   sequence,
                   cycles_completed,
                   cycles_remaining,
                   current_pricing_scheme_version,
                   total_cycles)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'tenure_type={self.tenure_type!r}, '
                f'sequence={self.sequence!r}, '
                f'cycles_completed={self.cycles_completed!r}, '
                f'cycles_remaining={(self.cycles_remaining if hasattr(self, "cycles_remaining") else None)!r}, '
                f'current_pricing_scheme_version={(self.current_pricing_scheme_version if hasattr(self, "current_pricing_scheme_version") else None)!r}, '
                f'total_cycles={(self.total_cycles if hasattr(self, "total_cycles") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'tenure_type={self.tenure_type!s}, '
                f'sequence={self.sequence!s}, '
                f'cycles_completed={self.cycles_completed!s}, '
                f'cycles_remaining={(self.cycles_remaining if hasattr(self, "cycles_remaining") else None)!s}, '
                f'current_pricing_scheme_version={(self.current_pricing_scheme_version if hasattr(self, "current_pricing_scheme_version") else None)!s}, '
                f'total_cycles={(self.total_cycles if hasattr(self, "total_cycles") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.auction_information import AuctionInformation
from paypalserversdk.models.cart_information import CartInformation
from paypalserversdk.models.incentive_information import IncentiveInformation
from paypalserversdk.models.payer_information import PayerInformation
from paypalserversdk.models.shipping_information import ShippingInformation
from paypalserversdk.models.store_information import StoreInformation
from paypalserversdk.models.transaction_information import TransactionInformation


class TransactionDetails(object):

    """Implementation of the 'Transaction Details' model.

    The transaction details.

    Attributes:
        transaction_info (TransactionInformation): The transaction information.
        payer_info (PayerInformation): The payer information.
        shipping_info (ShippingInformation): The shipping information.
        cart_info (CartInformation): The cart information.
        store_info (StoreInformation): The store information.
        auction_info (AuctionInformation): The auction information.
        incentive_info (IncentiveInformation): The incentive details.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "transaction_info": 'transaction_info',
        "payer_info": 'payer_info',
        "shipping_info": 'shipping_info',
        "cart_info": 'cart_info',
        "store_info": 'store_info',
        "auction_info": 'auction_info',
        "incentive_info": 'incentive_info'
    }

    _optionals = [
        'transaction_info',
        'payer_info',
        'shipping_info',
        'cart_info',
        'store_info',
        'auction_info',
        'incentive_info',
    ]

    def __init__(self,
                 transaction_info=APIHelper.SKIP,
                 payer_info=APIHelper.SKIP,
                 shipping_info=APIHelper.SKIP,
                 cart_info=APIHelper.SKIP,
                 store_info=APIHelper.SKIP,
                 auction_info=APIHelper.SKIP,
                 incentive_info=APIHelper.SKIP):
        """Constructor for the TransactionDetails class"""

        # Initialize members of the class
        if transaction_info is not APIHelper.SKIP:
            self.transaction_info = transaction_info 
        if payer_info is not APIHelper.SKIP:
            self.payer_info = payer_info 
        if shipping_info is not APIHelper.SKIP:
            self.shipping_info = shipping_info 
        if cart_info is not APIHelper.SKIP:
            self.cart_info = cart_info 
        if store_info is not APIHelper.SKIP:
            self.store_info = store_info 
        if auction_info is not APIHelper.SKIP:
            self.auction_info = auction_info 
        if incentive_info is not APIHelper.SKIP:
            self.incentive_info = incentive_info 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        transaction_info = TransactionInformation.from_dictionary(dictionary.get('transaction_info')) if 'transaction_info' in dictionary.keys() else APIHelper.SKIP
        payer_info = PayerInformation.from_dictionary(dictionary.get('payer_info')) if 'payer_info' in dictionary.keys() else APIHelper.SKIP
        shipping_info = ShippingInformation.from_dictionary(dictionary.get('shipping_info')) if 'shipping_info' in dictionary.keys() else APIHelper.SKIP
        cart_info = CartInformation.from_dictionary(dictionary.get('cart_info')) if 'cart_info' in dictionary.keys() else APIHelper.SKIP
        store_info = StoreInformation.from_dictionary(dictionary.get('store_info')) if 'store_info' in dictionary.keys() else APIHelper.SKIP
        auction_info = AuctionInformation.from_dictionary(dictionary.get('auction_info')) if 'auction_info' in dictionary.keys() else APIHelper.SKIP
        incentive_info = IncentiveInformation.from_dictionary(dictionary.get('incentive_info')) if 'incentive_info' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(transaction_info,
                   payer_info,
                   shipping_info,
                   cart_info,
                   store_info,
                   auction_info,
                   incentive_info)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'transaction_info={(self.transaction_info if hasattr(self, "transaction_info") else None)!r}, '
                f'payer_info={(self.payer_info if hasattr(self, "payer_info") else None)!r}, '
                f'shipping_info={(self.shipping_info if hasattr(self, "shipping_info") else None)!r}, '
                f'cart_info={(self.cart_info if hasattr(self, "cart_info") else None)!r}, '
                f'store_info={(self.store_info if hasattr(self, "store_info") else None)!r}, '
                f'auction_info={(self.auction_info if hasattr(self, "auction_info") else None)!r}, '
                f'incentive_info={(self.incentive_info if hasattr(self, "incentive_info") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'transaction_info={(self.transaction_info if hasattr(self, "transaction_info") else None)!s}, '
                f'payer_info={(self.payer_info if hasattr(self, "payer_info") else None)!s}, '
                f'shipping_info={(self.shipping_info if hasattr(self, "shipping_info") else None)!s}, '
                f'cart_info={(self.cart_info if hasattr(self, "cart_info") else None)!s}, '
                f'store_info={(self.store_info if hasattr(self, "store_info") else None)!s}, '
                f'auction_info={(self.auction_info if hasattr(self, "auction_info") else None)!s}, '
                f'incentive_info={(self.incentive_info if hasattr(self, "incentive_info") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class DisbursementMode(object):

    """Implementation of the 'Disbursement Mode' enum.

    The funds that are held on behalf of the merchant.

    Attributes:
        INSTANT: The funds are released to the merchant immediately.
        DELAYED: The funds are held for a finite number of days. The actual
            duration depends on the region and type of integration. You can
            release the funds through a referenced payout. Otherwise, the
            funds disbursed automatically after the specified duration.

    """
    INSTANT = 'INSTANT'

    DELAYED = 'DELAYED'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class SubscriptionPricingModel(object):

    """Implementation of the 'Subscription Pricing Model' enum.

    The pricing model for tiered plan. The `tiers` parameter is required.

    Attributes:
        VOLUME: A volume pricing model.
        TIERED: A tiered pricing model.

    """
    VOLUME = 'VOLUME'

    TIERED = 'TIERED'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.subscription_pricing_scheme import SubscriptionPricingScheme


class BillingCycleOverride(object):

    """Implementation of the 'Billing Cycle Override' model.

    The billing cycle details to override at subscription level. The
    subscription billing cycle definition has to adhere to the plan billing
    cycle definition.

    Attributes:
        pricing_scheme (SubscriptionPricingScheme): The pricing scheme details.
        sequence (int): The order in which this cycle is to run among other
            billing cycles. For example, a trial billing cycle has a
            `sequence` of `1` while a regular billing cycle has a `sequence`
            of `2`, so that trial cycle runs before the regular cycle.
        total_cycles (int): The number of times this billing cycle gets
            executed. Trial billing cycles can only be executed a finite
            number of times (value between 1 and 999 for total_cycles).
            Regular billing cycles can be executed infinite times (value of 0
            for total_cycles) or a finite number of times (value between 1 and
            999 for total_cycles).

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "sequence": 'sequence',
        "pricing_scheme": 'pricing_scheme',
        "total_cycles": 'total_cycles'
    }

    _optionals = [
        'pricing_scheme',
        'total_cycles',
    ]

    def __init__(self,
                 sequence=None,
                 pricing_scheme=APIHelper.SKIP,
                 total_cycles=APIHelper.SKIP):
        """Constructor for the BillingCycleOverride class"""

        # Initialize members of the class
        if pricing_scheme is not APIHelper.SKIP:
            self.pricing_scheme = pricing_scheme 
        self.sequence = sequence 
        if total_cycles is not APIHelper.SKIP:
            self.total_cycles = total_cycles 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        sequence = dictionary.get("sequence") if dictionary.get("sequence") else None
        pricing_scheme = SubscriptionPricingScheme.from_dictionary(dictionary.get('pricing_scheme')) if 'pricing_scheme' in dictionary.keys() else APIHelper.SKIP
        total_cycles = dictionary.get("total_cycles") if dictionary.get("total_cycles") else APIHelper.SKIP
        # Return an object of this model
        return cls(sequence,
                   pricing_scheme,
                   total_cycles)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'pricing_scheme={(self.pricing_scheme if hasattr(self, "pricing_scheme") else None)!r}, '
                f'sequence={self.sequence!r}, '
                f'total_cycles={(self.total_cycles if hasattr(self, "total_cycles") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'pricing_scheme={(self.pricing_scheme if hasattr(self, "pricing_scheme") else None)!s}, '
                f'sequence={self.sequence!s}, '
                f'total_cycles={(self.total_cycles if hasattr(self, "total_cycles") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class ItemCategory(object):

    """Implementation of the 'Item Category' enum.

    The item category type.

    Attributes:
        DIGITAL_GOODS: Goods that are stored, delivered, and used in their
            electronic format. This value is not currently supported for API
            callers that leverage the PayPal for Commerce Platform product.
        PHYSICAL_GOODS: A tangible item that can be shipped with proof of
            delivery.
        DONATION: A contribution or gift for which no good or service is
            exchanged, usually to a not for profit organization.

    """
    DIGITAL_GOODS = 'DIGITAL_GOODS'

    PHYSICAL_GOODS = 'PHYSICAL_GOODS'

    DONATION = 'DONATION'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.payment_method_preference import PaymentMethodPreference
from paypalserversdk.models.stored_payment_source import StoredPaymentSource


class OrderApplicationContext(object):

    """Implementation of the 'Order Application Context' model.

    Customizes the payer experience during the approval process for the
    payment with PayPal. Note: Partners and Marketplaces might configure
    brand_name and shipping_preference during partner account setup, which
    overrides the request values.

    Attributes:
        brand_name (str): DEPRECATED. The label that overrides the business
            name in the PayPal account on the PayPal site. The fields in
            `application_context` are now available in the
            `experience_context` object under the `payment_source` which
            supports them (eg.
            `payment_source.paypal.experience_context.brand_name`). Please
            specify this field in the `experience_context` object instead of
            the `application_context` object.
        locale (str): DEPRECATED. The BCP 47-formatted locale of pages that
            the PayPal payment experience shows. PayPal supports a
            five-character code. For example, `da-DK`, `he-IL`, `id-ID`,
            `ja-JP`, `no-NO`, `pt-BR`, `ru-RU`, `sv-SE`, `th-TH`, `zh-CN`,
            `zh-HK`, or `zh-TW`.  The fields in `application_context` are now
            available in the `experience_context` object under the
            `payment_source` which supports them (eg.
            `payment_source.paypal.experience_context.locale`). Please specify
            this field in the `experience_context` object instead of the
            `application_context` object.
        landing_page (OrderApplicationContextLandingPage): DEPRECATED.
            DEPRECATED. The type of landing page to show on the PayPal site
            for customer checkout.  The fields in `application_context` are
            now available in the `experience_context` object under the
            `payment_source` which supports them (eg.
            `payment_source.paypal.experience_context.landing_page`). Please
            specify this field in the `experience_context` object instead of
            the `application_context` object.
        shipping_preference (OrderApplicationContextShippingPreference):
            DEPRECATED. DEPRECATED. The shipping preference: Displays the
            shipping address to the customer. Enables the customer to choose
            an address on the PayPal site. Restricts the customer from
            changing the address during the payment-approval process. .  The
            fields in `application_context` are now available in the
            `experience_context` object under the `payment_source` which
            supports them (eg.
            `payment_source.paypal.experience_context.shipping_preference`).
            Please specify this field in the `experience_context` object
            instead of the `application_context` object.
        user_action (OrderApplicationContextUserAction): DEPRECATED.
            Configures a Continue or Pay Now checkout flow.  The fields in
            `application_context` are now available in the
            `experience_context` object under the `payment_source` which
            supports them (eg.
            `payment_source.paypal.experience_context.user_action`). Please
            specify this field in the `experience_context` object instead of
            the `application_context` object.
        payment_method (PaymentMethodPreference): DEPRECATED. The customer and
            merchant payment preferences. The fields in `application_context`
            are now available in the `experience_context` object under the
            `payment_source` which supports them (eg.
            `payment_source.paypal.experience_context.payment_method_selected`)
            . Please specify this field in the `experience_context` object
            instead of the `application_context` object..
        return_url (str): DEPRECATED. The URL where the customer is redirected
            after the customer approves the payment. The fields in
            `application_context` are now available in the
            `experience_context` object under the `payment_source` which
            supports them (eg.
            `payment_source.paypal.experience_context.return_url`). Please
            specify this field in the `experience_context` object instead of
            the `application_context` object.
        cancel_url (str): DEPRECATED. The URL where the customer is redirected
            after the customer cancels the payment. The fields in
            `application_context` are now available in the
            `experience_context` object under the `payment_source` which
            supports them (eg.
            `payment_source.paypal.experience_context.cancel_url`). Please
            specify this field in the `experience_context` object instead of
            the `application_context` object.
        stored_payment_source (StoredPaymentSource): DEPRECATED. Provides
            additional details to process a payment using a `payment_source`
            that has been stored or is intended to be stored (also referred to
            as stored_credential or card-on-file). Parameter compatibility:
            `payment_type=ONE_TIME` is compatible only with
            `payment_initiator=CUSTOMER`. `usage=FIRST` is compatible only
            with `payment_initiator=CUSTOMER`.
            `previous_transaction_reference` or
            `previous_network_transaction_reference` is compatible only with
            `payment_initiator=MERCHANT`. Only one of the parameters -
            `previous_transaction_reference` and
            `previous_network_transaction_reference` - can be present in the
            request. .  The fields in `stored_payment_source` are now
            available in the `stored_credential` object under the
            `payment_source` which supports them (eg.
            `payment_source.card.stored_credential.payment_initiator`). Please
            specify this field in the `payment_source` object instead of the
            `application_context` object.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "brand_name": 'brand_name',
        "locale": 'locale',
        "landing_page": 'landing_page',
        "shipping_preference": 'shipping_preference',
        "user_action": 'user_action',
        "payment_method": 'payment_method',
        "return_url": 'return_url',
        "cancel_url": 'cancel_url',
        "stored_payment_source": 'stored_payment_source'
    }

    _optionals = [
        'brand_name',
        'locale',
        'landing_page',
        'shipping_preference',
        'user_action',
        'payment_method',
        'return_url',
        'cancel_url',
        'stored_payment_source',
    ]

    def __init__(self,
                 brand_name=APIHelper.SKIP,
                 locale=APIHelper.SKIP,
                 landing_page='NO_PREFERENCE',
                 shipping_preference='GET_FROM_FILE',
                 user_action='CONTINUE',
                 payment_method=APIHelper.SKIP,
                 return_url=APIHelper.SKIP,
                 cancel_url=APIHelper.SKIP,
                 stored_payment_source=APIHelper.SKIP):
        """Constructor for the OrderApplicationContext class"""

        # Initialize members of the class
        if brand_name is not APIHelper.SKIP:
            self.brand_name = brand_name 
        if locale is not APIHelper.SKIP:
            self.locale = locale 
        self.landing_page = landing_page 
        self.shipping_preference = shipping_preference 
        self.user_action = user_action 
        if payment_method is not APIHelper.SKIP:
            self.payment_method = payment_method 
        if return_url is not APIHelper.SKIP:
            self.return_url = return_url 
        if cancel_url is not APIHelper.SKIP:
            self.cancel_url = cancel_url 
        if stored_payment_source is not APIHelper.SKIP:
            self.stored_payment_source = stored_payment_source 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        brand_name = dictionary.get("brand_name") if dictionary.get("brand_name") else APIHelper.SKIP
        locale = dictionary.get("locale") if dictionary.get("locale") else APIHelper.SKIP
        landing_page = dictionary.get("landing_page") if dictionary.get("landing_page") else 'NO_PREFERENCE'
        shipping_preference = dictionary.get("shipping_preference") if dictionary.get("shipping_preference") else 'GET_FROM_FILE'
        user_action = dictionary.get("user_action") if dictionary.get("user_action") else 'CONTINUE'
        payment_method = PaymentMethodPreference.from_dictionary(dictionary.get('payment_method')) if 'payment_method' in dictionary.keys() else APIHelper.SKIP
        return_url = dictionary.get("return_url") if dictionary.get("return_url") else APIHelper.SKIP
        cancel_url = dictionary.get("cancel_url") if dictionary.get("cancel_url") else APIHelper.SKIP
        stored_payment_source = StoredPaymentSource.from_dictionary(dictionary.get('stored_payment_source')) if 'stored_payment_source' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(brand_name,
                   locale,
                   landing_page,
                   shipping_preference,
                   user_action,
                   payment_method,
                   return_url,
                   cancel_url,
                   stored_payment_source)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'brand_name={(self.brand_name if hasattr(self, "brand_name") else None)!r}, '
                f'locale={(self.locale if hasattr(self, "locale") else None)!r}, '
                f'landing_page={(self.landing_page if hasattr(self, "landing_page") else None)!r}, '
                f'shipping_preference={(self.shipping_preference if hasattr(self, "shipping_preference") else None)!r}, '
                f'user_action={(self.user_action if hasattr(self, "user_action") else None)!r}, '
                f'payment_method={(self.payment_method if hasattr(self, "payment_method") else None)!r}, '
                f'return_url={(self.return_url if hasattr(self, "return_url") else None)!r}, '
                f'cancel_url={(self.cancel_url if hasattr(self, "cancel_url") else None)!r}, '
                f'stored_payment_source={(self.stored_payment_source if hasattr(self, "stored_payment_source") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'brand_name={(self.brand_name if hasattr(self, "brand_name") else None)!s}, '
                f'locale={(self.locale if hasattr(self, "locale") else None)!s}, '
                f'landing_page={(self.landing_page if hasattr(self, "landing_page") else None)!s}, '
                f'shipping_preference={(self.shipping_preference if hasattr(self, "shipping_preference") else None)!s}, '
                f'user_action={(self.user_action if hasattr(self, "user_action") else None)!s}, '
                f'payment_method={(self.payment_method if hasattr(self, "payment_method") else None)!s}, '
                f'return_url={(self.return_url if hasattr(self, "return_url") else None)!s}, '
                f'cancel_url={(self.cancel_url if hasattr(self, "cancel_url") else None)!s}, '
                f'stored_payment_source={(self.stored_payment_source if hasattr(self, "stored_payment_source") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class PricingModel(object):

    """Implementation of the 'Pricing Model' enum.

    The pricing model for the billing cycle.

    Attributes:
        FIXED: A fixed pricing scheme where the customer is charged a fixed
            amount.
        VARIABLE: A variable pricing scheme where the customer is charged a
            variable amount.
        AUTO_RELOAD: A auto-reload pricing scheme where the customer is
            charged a fixed amount for reload.

    """
    FIXED = 'FIXED'

    VARIABLE = 'VARIABLE'

    AUTO_RELOAD = 'AUTO_RELOAD'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.sepa_debit_experience_context import SepaDebitExperienceContext


class SepaDebitRequest(object):

    """Implementation of the 'SEPA Debit Request' model.

    An API resource denoting a request to securely store a SEPA Debit.

    Attributes:
        experience_context (SepaDebitExperienceContext): Customizes the payer
            experience during the approval process for the SEPA Debit payment.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "experience_context": 'experience_context'
    }

    _optionals = [
        'experience_context',
    ]

    def __init__(self,
                 experience_context=APIHelper.SKIP):
        """Constructor for the SepaDebitRequest class"""

        # Initialize members of the class
        if experience_context is not APIHelper.SKIP:
            self.experience_context = experience_context 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        experience_context = SepaDebitExperienceContext.from_dictionary(dictionary.get('experience_context')) if 'experience_context' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(experience_context)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'experience_context={(self.experience_context if hasattr(self, "experience_context") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'experience_context={(self.experience_context if hasattr(self, "experience_context") else None)!s})')



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper


class CheckoutOption(object):

    """Implementation of the 'Checkout Option' model.

    A checkout option as a name-and-value pair.

    Attributes:
        checkout_option_name (str): The checkout option name, such as `color`
            or `texture`.
        checkout_option_value (str): The checkout option value. For example,
            the checkout option `color` might be `blue` or `red` while the
            checkout option `texture` might be `smooth` or `rippled`.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "checkout_option_name": 'checkout_option_name',
        "checkout_option_value": 'checkout_option_value'
    }

    _optionals = [
        'checkout_option_name',
        'checkout_option_value',
    ]

    def __init__(self,
                 checkout_option_name=APIHelper.SKIP,
                 checkout_option_value=APIHelper.SKIP):
        """Constructor for the CheckoutOption class"""

        # Initialize members of the class
        if checkout_option_name is not APIHelper.SKIP:
            self.checkout_option_name = checkout_option_name 
        if checkout_option_value is not APIHelper.SKIP:
            self.checkout_option_value = checkout_option_value 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if not isinstance(dictionary, dict) or dictionary is None:
            return None

        # Extract variables from the dictionary
        checkout_option_name = dictionary.get("checkout_option_name") if dictionary.get("checkout_option_name") else APIHelper.SKIP
        checkout_option_value = dictionary.get("checkout_option_value") if dictionary.get("checkout_option_value") else APIHelper.SKIP
        # Return an object of this model
        return cls(checkout_option_name,
                   checkout_option_value)

    def __repr__(self):
        return (f'{self.__class__.__name__}('
                f'checkout_option_name={(self.checkout_option_name if hasattr(self, "checkout_option_name") else None)!r}, '
                f'checkout_option_value={(self.checkout_option_value if hasattr(self, "checkout_option_value") else None)!r})')

    def __str__(self):
        return (f'{self.__class__.__name__}('
                f'checkout_option_name={(self.checkout_option_name if hasattr(self, "checkout_option_name") else None)!s}, '
                f'checkout_option_value={(self.checkout_option_value if hasattr(self, "checkout_option_value") else None)!s})')
